#!/bin/bash


#参数设置
CURRENT_DIR=$(cd `dirname $0`; pwd)
ip_address=$1
kungate_server_port=$2
kundb_user=$3
kundb_passwd=$4
kungate_debug_port=15001


cp -r ../tools/ ${CURRENT_DIR}/

kundb_connet_cmd="${CURRENT_DIR}/tools/mysql -h $ip_address -P$kungate_server_port -u$kundb_user -p${kundb_passwd}" 
kundb_check_result=$CURRENT_DIR/result_tmp.csv
touch -c $kundb_check_result

function gateway_port_nc_check()
{
bash -c "${CURRENT_DIR}/tools/nc -w 10 -z $ip_address $kungate_server_port" &>/dev/null; res=$?
if [ $res -eq 0 ];then
    echo "connect to kundb gateway successful"
    echo "start checking...."
elif [ $res -eq 127 ];then
    echo "Unable to execute the 'nc' command. You can execute ldd ./nc in the tools/ directory to checks what libraries missing"
    echo "Notice: If you confirm that the Kungate port is OK, you can comment this scrit '#gateway_port_nc_check' at line 34 to skip port check" 
    exit 1
elif [  $res -ne 0 -o $res -ne 127 ];then
    echo "error: cannot connect kundb gateway service nodeport,please check first!"
    exit 1
fi
}
gateway_port_nc_check

function gateway_account_check()
{
$kundb_connet_cmd -e "show databases;" &>/dev/null; res=$?
if [ $res -eq 0 ];then
    echo "kundb账号检测,true,true,ok,账号密码可以正常访问" >> $kundb_check_result 
    connect_result=true
else
    echo "kundb账号检测,flase,true,no,账号密码无法访问请校验" >> $kundb_check_result
    connect_result=false 
fi
}


#查看shard数
get_shard_num="show kundb_shards" 
#查看大小写参数，默认修改后无法
get_lower_case="show variables like 'lower_case_table_names';"
#查看gms连接的数量
get_gms_connet="select * from performance_schema.replication_group_members;"
#查看是否开启主备同步
get_replicaion_group_mem="select * from performance_schema.replication_group_members;"
#查看主备同步方式
get_semi_sync_replica_enable="SHOW variables LIKE '%rpl_semi_sync_replica_enabled%';"
get_semi_sync_source_enable="SHOW variables LIKE '%rpl_semi_sync_source_enabled%';"
#查看是否开启TSO
get_tso_enable="SHOW variables LIKE 'innodb_enable_tso';"
#查看sql方言
get_sql_dialect="show sql_dialect;"
#查看binlog超时时间
get_binlog_status="SHOW variables LIKE '%binlog_expire_logs_seconds%';"
#查看mysql版本
get_mysql_version="select version();"
#查看innodb参数
get_innodb_status="show variables like 'innodb%';"


function kundb_sql_check_kundbmode()
{
result_get_kundbmode=`$kundb_connet_cmd -e "${get_gms_connet}"` 
echo $result_get_kundbmode
if [ -z "$result_get_kundbmode"  ];then
    export kundbmode="主备模式"
else
    export kundbmode="paxos"
fi
echo "kundb主备模式,$kundbmode,,,默认为主备模式(paxos需要修改配置)" >> $kundb_check_result
}


function kundb_sql_check_lowercase()
{
#检测是否开启大小写
result_get_lowercase=`$kundb_connet_cmd -e "${get_lower_case}" -N -s |awk '{print $2}'` 
if [ -z $result_get_lowercase  ];then  
    echo "检测是否开启大小写,$result_get_lowercase,0 or 1,no,无法获取大小写信息" >> $kundb_check_result
else
    echo "检测是否开启大小写,$result_get_lowercase,0 or 1,ok,0开启1不开启" >> $kundb_check_result
fi
}


function kundb_sql_check_shardnum()
{
result_get_shardnum=`$kundb_connet_cmd -e "${get_shard_num}" -N -s  |grep kundb |wc -l`
if [ $result_get_shardnum -ge  1 ];then 
    echo "shard数量,$result_get_shardnum,>=1,ok,shard至少为1个" >> $kundb_check_result
else
    echo "shard数量,$result_get_shardnum,>=1,no,shard至少为1个" >> $kundb_check_result
fi
}


function kundb_sql_check_dialect()
{
result_get_dialect=`$kundb_connet_cmd -e "${get_sql_dialect}" -N -s`
if [ -z result_get_dialect  ];then 
    echo "kundb当前方言,$result_get_dialect,MySQL or Oracle",no,无法获取kundb方言信息" >> $kundb_check_result
else
    echo "kundb当前方言,$result_get_dialect,MySQL or Oracle",ok,目前支持MySQL or Oracle" >> $kundb_check_result
fi
}


function kundb_sql_check_mysqlversion()
{
result_get_mysqlversion=`$kundb_connet_cmd -e "${get_mysql_version}" -N -s`
echo "源数据库版本,$result_get_mysqlversion,8.x,ok,如果是mysql57，不能使用kuntool的新版本进行导出数据" >> $kundb_check_result
}


function kundb_sql_check_snyc_switch()
{
result_get_replica_enable=`$kundb_connet_cmd -e "${get_semi_sync_replica_enable}" -N -s |awk '{print $2}'`
result_get_source_enable=`$kundb_connet_cmd -e "${get_semi_sync_source_enable}" -N -s |awk '{print $2}'`
if [ -z $result_get_source_enable ];then
    echo "echo "源库是否开启半同步参数,$result_get_source_enable,ON,no,无法获取参数信息" >> $kundb_check_result"
else
    if [ "$result_get_source_enable" == "ON" ];then
        echo "源库是否开启半同步参数,$result_get_source_enable,ON,ok,已开启同步" >> $kundb_check_result
    else
        echo "源库是否开启半同步参数,$result_get_source_enable,ON,no,未开启同步" >> $kundb_check_result
    fi
fi

if [ -z $result_get_replica_enable ];then
    echo "echo "源库是否开启半同步参数,$result_get_replica_enable,ON,no,无法获取参数信息" >> $kundb_check_result"
else
    if [ "$result_get_replica_enable" == "ON" ];then
        echo "从库是否开启半同步参数,$result_get_replica_enable,ON,ok,已开启同步" >> $kundb_check_result
    else
        echo "从库是否开启半同步参数,$result_get_replica_enable,ON,no,未开启同步" >> $kundb_check_result
    fi
fi
}


function kundb_sql_check_binlog_status()
{
result_get_binlog_status=`$kundb_connet_cmd -e "${get_binlog_status}" -N -s|awk '{print $2}'`
if [[ $result_get_binlog_status -ge 86400 ]] && [[ $result_get_binlog_status -le 864000 ]];then
    echo "binlog过期时间,$result_get_binlog_status",86400≤X≤864000,ok,rotate时间不能小于1天，大于10天 >> $kundb_check_result
else
    echo "binlog过期时间,$result_get_binlog_status",86400≤X≤864000,no,rotate时间不能小于1天，大于10天 >> $kundb_check_result
fi

}


function kundb_sql_check_tso()
{
result_get_tso_status=`$kundb_connet_cmd -e "${get_tso_enable}" -N -s |awk '{print $2}'` 
if [ -z $result_get_lowercase  ];then  
    echo "检查是否开启了2PC（TSO）,$result_get_tso_status,0 or 1,no,无法获取是否开启TSO" >> $kundb_check_result
else
    echo "检查是否开启了2PC（TSO）,$result_get_tso_status,ON or OFF,ok,ON表示开了TSO， 否则没有打开" >> $kundb_check_result
fi
}



function kundb_queryz_web_check()
{
#queryz_nodeport=`kubectl get svc -A |grep $kungate_server_port |awk '{print $6}'|tr -s "," "\n"|grep 15001|cut -f 2 -d ":" |cut -f 1 -d "/"`
queryz_nodeport=$kungate_debug_port
if [ -z $queryz_nodeport ];then
    echo "慢sql页面健康状态,异常,正常,no,无法获取端口信息" >> $kundb_check_result
else
    queryz_url="http://$ip_address:$queryz_nodeport/queryz"
    queryz_url_res_code=`curl -o /dev/null -s -w %{http_code} $queryz_url`
    if [ $queryz_url_res_code -eq 200 ];then
        echo "慢sql页面健康状态,正常,正常,ok,请查看$queryz_url" >> $kundb_check_result
    else
        echo "慢sql页面健康状态,异常,正常,no,请检测$queryz_url" >> $kundb_check_result
    fi
fi
}


function get_innodb_conf()
{
echo -e "\n" >> $kundb_check_result
echo "innodb参数,值" >> $kundb_check_result
$kundb_connet_cmd -e "${get_innodb_status}" -N -s |grep -E "innodb_read_io_threads|innodb_write_io_threads|innodb_lock_wait_timeout|innodb_buffer_pool_size|innodb_buffer_pool_instances|innodb_flush_method|innodb_log_buffer_size|innodb_log_file_size|net_read_timeout|net_write_timeout" > innodb_conf_tmp_table
#$kundb_connet_cmd -e "${get_innodb_status}" -N -s  > innodb_conf_tmp_table
cat innodb_conf_tmp_table |while read line
do   
innodb_conf=`echo $line |awk '{print $1}'`
innodb_value=`echo $line |awk '{print $2}'`
echo "$innodb_conf,$innodb_value" >> $kundb_check_result
done
rm -rf innodb_conf_tmp_table
}


echo "kundb健康检测,当前值,期望值,是否健康,注释" >> $kundb_check_result
gateway_account_check
echo -e "\n" >> $kundb_check_result

if [ $connect_result == "true" ];then
    echo "kundb数据库检测,当前值,期望值,是否健康,注释" >> $kundb_check_result
    kundb_sql_check_kundbmode
    kundb_sql_check_lowercase
    kundb_sql_check_shardnum
    kundb_sql_check_dialect
    kundb_sql_check_mysqlversion
    kundb_sql_check_snyc_switch
    kundb_sql_check_binlog_status
    kundb_sql_check_tso
    kundb_queryz_web_check
    get_innodb_conf
else
    echo "Unable to connect to kundb, cannot exec sql check results"
fi

rm -rf tools
iconv -f UTF-8 -t GBK $kundb_check_result -o kundb_check.csv
rm -rf $kundb_check_result
echo "---------------------------------------"
echo "please get ckeck result in $CURRENT_DIR/kundb_check.csv"
echo "---------------------------------------"